@@ -1,9 +1,9 @@
 ;;; supercite.el --- minor mode for citing mail and news replies
 
-;; Copyright (C) 1993, 1997 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1997, 2001 Free Software Foundation, Inc.
 
 ;; Author: 1993 Barry A. Warsaw, Century Computing, Inc. <bwarsaw@cen.com>
-;; Maintainer:    supercite-help@python.org
+;; Maintainer:    Mark Senn <mds@ecn.purdue.edu>
 ;; Created:       February 1993
 ;; Last Modified: 1993/09/22 18:58:46
 ;; Keywords: mail, news
@@ -28,14 +28,17 @@
 ;; Boston, MA 02111-1307, USA.
 
 ;; LCD Archive Entry
-;; supercite|Barry A. Warsaw|supercite-help@anthem.nlm.nih.gov
+;; supercite|Barry A. Warsaw|supercite-help@python.org
 ;; |Mail and news reply citation package
 ;; |1993/09/22 18:58:46|3.1|
 
-;; Code:
+;;; Commentary:
+
+;;; Code:
 
 
 (require 'regi)
+(require 'sendmail)	;; For mail-header-end.
 
 ;; start user configuration variables
 ;; vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
@@ -102,14 +105,14 @@ This should NOT have a leading `^' character."
 
 ;; Nemacs and Mule users note: please see the texinfo manual for
 ;; suggestions on setting these variables.
-(defcustom sc-citation-root-regexp "[-._a-zA-Z0-9]*"
+(defcustom sc-citation-root-regexp "[-._[:alnum:]]*"
   "*Regexp describing variable root part of a citation for a cited line.
 This should NOT have a leading `^' character.  See also
 `sc-citation-nonnested-root-regexp'."
   :type 'regexp
   :group 'supercite-cite)
 
-(defcustom sc-citation-nonnested-root-regexp "[-._a-zA-Z0-9]+"
+(defcustom sc-citation-nonnested-root-regexp "[-._[:alnum:]]+"
   "*Regexp describing the variable root part of a nested citation.
 This should NOT have a leading `^' character.  This variable is
 related to `sc-citation-root-regexp' but whereas that variable
@@ -253,9 +256,9 @@ The gathering of attribution information is not affected by the value
 of this variable.  The number of lines in the region is calculated
 *after* all mail headers are removed.  This variable is only consulted
 during the initial citing via `sc-cite-original'."
-  :type '(choice (const :tag "always cite" t)
-		 (const :tag "do not cite" nil)
-		 (integer :tag "citation threshold"))
+  :type '(choice (const :tag "do not cite" nil)
+		 (integer :tag "citation threshold")
+		 (other :tag "always cite" t))
   :group 'supercite-cite)
 
 (defcustom sc-confirm-always-p t
@@ -418,9 +421,10 @@ and the return value must be a string, which is used as the selected
 attribution.  Note that the variable `sc-preferred-attribution-list'
 must contain an element of the string \"sc-consult\" for this variable
 to be consulted during attribution selection."
-  :type '(repeat (list symbol (repeat (cons regexp
-					    (choice (repeat (repeat sexp))
-						    symbol)))))
+  :type '(repeat (list string
+		       (repeat (cons regexp
+				     (choice (sexp :tag "List to eval")
+					     string)))))
   :group 'supercite-attr)
 
 (defcustom sc-attribs-preselect-hook nil
@@ -498,7 +502,7 @@ string."
 
 (defconst sc-version "3.1"
   "Supercite version number.")
-(defconst sc-help-address "supercite-help@anthem.nlm.nih.gov"
+(defconst sc-help-address "bug-supercite@gnu.org"
   "Address accepting submissions of bug reports.")
 
 (defvar sc-mail-info nil
@@ -509,7 +513,8 @@ string."
 (defconst sc-emacs-features
   (let ((version 'v18)
 	(flavor  'GNU))
-    (if (string= (substring emacs-version 0 2) "19")
+    (if (not
+	 (string= (substring emacs-version 0 2) "18"))
 	(setq version 'v19))
     (if (string-match "Lucid" emacs-version)
 	(setq flavor 'Lucid))
@@ -518,9 +523,9 @@ string."
   "A list describing what version of Emacs we're running on.
 Known flavors are:
 
-All GNU18's: (v18 GNU)
-FSF19.x    : (v19 GNU)
-Lucid19.x  : (v19 Lucid)")
+Emacs 18           : (v18 GNU)
+Emacs 19 or later  : (v19 GNU)
+Lucid 19 or later  : (v19 Lucid)")
 
 
 (defvar sc-tmp-nested-regexp nil
@@ -646,8 +651,7 @@ In version 18, the HISTORY argument is ignored."
   "Compatibility between Emacs 18 and 19 `read-string'.
 In version 18, the HISTORY argument is ignored."
   (if (memq 'v19 sc-emacs-features)
-      ;; maybe future versions will take a `history' argument:
-      (read-string prompt initial-contents)
+      (read-string prompt initial-contents history)
     (read-string prompt initial-contents)))
 
 (if (fboundp 'match-string)
@@ -713,7 +717,7 @@ the list should be unique."
 	      (setq event (read-char)))
 	     ((memq 'Lucid sc-emacs-features)
 	      (next-command-event event))
-	     (t				; must be FSF19
+	     (t				; must be Emacs 19
 	      (setq event (read-event))))
 	    (prog1 quit-flag (setq quit-flag nil)))
 	  (progn
@@ -797,6 +801,8 @@ the list should be unique."
     (end                          (setq sc-mail-headers-end (point))))
   "Regi frame for glomming mail header information.")
 
+(eval-when-compile (defvar curline))	; dynamic bondage
+
 ;; regi functions
 (defun sc-mail-fetch-field (&optional attribs-p)
   "Insert a key and value into `sc-mail-info' alist.
@@ -1007,7 +1013,7 @@ AUTHOR is the author's name (which is removed from the address)."
 		   (= (aref address (1- (length address))) ?>))
 	      (substring address 1 (1- (length address)))
 	    address))
-      (if (string-match "[-a-zA-Z0-9!@%._]+" from 0)
+      (if (string-match "[-[:alnum:]!@%._]+" from 0)
 	  (sc-submatch 0 from)
 	"")
       )))
@@ -1054,7 +1060,7 @@ This should be the author's full name minus an optional title."
 	  (sc-name-substring
 	   from (string-match "\".*\"" from 0) (match-end 0) 1)
 	  (sc-name-substring
-	   from (string-match "\\([-.a-zA-Z0-9_]+\\s +\\)+<" from 0)
+	   from (string-match "\\([-.[:alnum:]_]+\\s +\\)+<" from 0)
 	   (match-end 1) 0)
 	  (sc-attribs-emailname from))))
     ;; strip off any leading or trailing whitespace
@@ -1577,8 +1583,7 @@ non-nil."
   (if (not (bobp))
       (if (and (eolp)
 	       (progn (forward-line -1)
-		      (or (looking-at
-			   (concat "^" (regexp-quote mail-header-separator) "$"))
+		      (or (= (point) (mail-header-end))
 			  (and (eq major-mode 'mh-letter-mode)
 			       (mh-in-header-p)))))
 	  (progn (forward-line)
@@ -2010,7 +2015,7 @@ before, and `sc-post-hook' is run after the guts of this function."
   ;; this function gets automatically called. we want point to be a
   ;; mark so any deleting before point works properly
   (let* ((zmacs-regions nil)		; for Lemacs
-	 (mark-active t)		; for FSFmacs
+	 (mark-active t)		; for Emacs
 	 (point (point-marker))
 	 (mark  (copy-marker (mark-marker))))
   
@@ -2092,7 +2097,7 @@ cited."
 	    (looking-at "^[ \t]*$")
 	    (consp arg))
 	(insert (sc-mail-field "sc-citation"))
-      (error "Line is already cited."))))
+      (error "Line is already cited"))))
 
 (defun sc-version (arg)
   "Echo the current version of Supercite in the minibuffer.
